from species.analysis.fit_model import FitModel

from species.analysis.fit_spectrum import FitSpectrum

from species.analysis.photometry import SyntheticPhotometry, \
                                        apparent_to_absolute, \
                                        multi_photometry


from species.read.read_calibration import ReadCalibration

from species.read.read_filter import ReadFilter

from species.read.read_model import ReadModel, \
                                    get_mass, \
                                    add_luminosity

from species.read.read_spectrum import ReadSpectrum, \
                                       get_planck

from species.read.read_color import ReadColorMagnitude, \
                                    ReadColorColor

from species.read.read_object import ReadObject

from species.core.box import open_box, \
                             create_box, \
                             SpectrumBox, \
                             PhotometryBox

from species.core.setup import SpeciesInit

from species.data.database import Database

from species.data.queries import get_distance

from species.plot.plot_mcmc import plot_posterior, \
                                   plot_walkers

from species.plot.plot_color import plot_color_magnitude, plot_color_color

from species.plot.plot_spectrum import plot_spectrum

__author__ = 'Tomas Stolker'
__license__ = 'GPLv3'
__version__ = '0.0.6'
__maintainer__ = 'Tomas Stolker'
__email__ = 'tomas.stolker@phys.ethz.ch'
__status__ = 'Development'



'''
Box module.
'''


def open_box(box):
    '''
    return:
    '''

    for item in box.__dict__.keys():
        sys.stdout.write(str(item)+' = '+str(box.__dict__[item])+'\n')
        sys.stdout.flush()

def create_box(boxtype, **kwargs):
    '''
    :return:
    '''

    if boxtype == 'colormag':
        box = ColorMagBox()
        box.library = kwargs['library']
        box.object_type = kwargs['object_type']
        box.filters_color = kwargs['filters_color']
        box.filter_mag = kwargs['filter_mag']
        box.color = kwargs['color']
        box.magnitude = kwargs['magnitude']
        box.sptype = kwargs['sptype']

    if boxtype == 'colorcolor':
        box = ColorColorBox()
        box.library = kwargs['library']
        box.object_type = kwargs['object_type']
        box.filters = kwargs['filters']
        box.color1 = kwargs['color1']
        box.color2 = kwargs['color2']
        box.sptype = kwargs['sptype']

    elif boxtype == 'model':
        box = ModelBox()
        box.model = kwargs['model']
        box.wavelength = kwargs['wavelength']
        box.flux = kwargs['flux']
        box.parameters = kwargs['parameters']

    elif boxtype == 'object':
        box = ObjectBox()
        box.name = kwargs['name']
        box.filter = kwargs['filter']
        box.magnitude = kwargs['magnitude']
        box.flux = kwargs['flux']
        box.distance = kwargs['distance']

    elif boxtype == 'photometry':
        box = PhotometryBox()
        box.name = kwargs['name']
        box.wavelength = kwargs['wavelength']
        box.flux = kwargs['flux']

    elif boxtype == 'samples':
        box = SamplesBox()
        box.spectrum = kwargs['spectrum']
        box.parameters = kwargs['parameters']
        box.samples = kwargs['samples']
        box.chisquare = kwargs['chisquare']

    elif boxtype == 'spectrum':
        box = SpectrumBox()
        box.spectrum = kwargs['spectrum']
        box.wavelength = kwargs['wavelength']
        box.flux = kwargs['flux']
        box.name = kwargs['name']
        box.simbad = kwargs['simbad']
        box.sptype = kwargs['sptype']
        box.distance = kwargs['distance']

    elif boxtype == 'synphot':
        box = SynphotBox()
        box.name = kwargs['name']
        box.flux = kwargs['flux']

    return box


class ColorMagBox:
    '''
    Text
    '''

    def __init__(self):
        '''
        :return:
        '''

        self.library = None
        self.object_type = None
        self.filters_color = None
        self.filter_mag = None
        self.color = None
        self.magnitude = None
        self.sptype = None


class ColorColorBox:
    '''
    Text
    '''

    def __init__(self):
        '''
        :return:
        '''

        self.library = None
        self.object_type = None
        self.filters = None
        self.color1 = None
        self.color2 = None
        self.sptype = None


class ModelBox:
    '''
    Text
    '''

    def __init__(self):
        '''
        :return:
        '''

        self.model = None
        self.type = None
        self.wavelength = None
        self.flux = None
        self.parameters = None


class ObjectBox:
    '''
    Text
    '''

    def __init__(self):
        '''
        :return:
        '''

        self.name = None
        self.filter = None
        self.magnitude = None
        self.flux = None
        self.distance = None


class PhotometryBox:
    '''
    Text
    '''

    def __init__(self):
        '''
        :return:
        '''

        self.name = None
        self.wavelength = None
        self.flux = None


class SamplesBox:
    '''
    Text
    '''

    def __init__(self):
        '''
        :return:
        '''

        self.spectrum = None
        self.parameters = None
        self.samples = None
        self.chisquare = None


class SpectrumBox:
    '''
    Text
    '''

    def __init__(self):
        '''
        :return:
        '''

        self.spectrum = None
        self.wavelength = None
        self.flux = None
        self.name = None
        self.simbad = None
        self.sptype = None
        self.distance = None


class SynphotBox:
    '''
    Text
    '''

    def __init__(self):
        '''
        :return:
        '''

        self.name = None
        self.flux = None



'''
Read module.
'''

import os
import configparser

import h5py
import numpy as np


class ReadResult:
    '''
    Text
    '''

    def __init__(self,
                 res_type,
                 tag):
        '''
        :param res_type: Result type.
        :type res_type: str
        :param tag: Database tag.
        :type tag: str

        :return: None
        '''

        self.res_type = res_type
        self.tag = tag

        config_file = os.path.join(os.getcwd(), 'species_config.ini')

        config = configparser.ConfigParser()
        config.read_file(open(config_file))

        self.database = config['species']['database']

    def get_chisquare(self, fix=None):
        '''
        :param fix:
        :type fix: dict

        :return: Grid points and reduced chi-square values.
        :rtype: numpy.ndarray, numpy.ndarray
        '''

        h5_file = h5py.File(self.database, 'r')

        dset = h5_file['results/chisquare/'+self.tag]
        nparam = int(dset.attrs['nparam'])

        points = {}
        for i in range(nparam):
            param = dset.attrs['parameter'+str(i)]
            values = np.asarray(h5_file['results/chisquare/'+self.tag+'/'+param])
            points[param] = values

        chisquare = np.asarray(h5_file['results/chisquare/'+self.tag+'/chisquare'])

        h5_file.close()

        if fix:
            if nparam == 4:
                indices = [slice(0, chisquare.shape[0]),
                           slice(0, chisquare.shape[1]),
                           slice(0, chisquare.shape[2]),
                           slice(0, chisquare.shape[3])]

            for item in fix:
                param_index = list(points.keys()).index(item)
                value_index = min(enumerate(points[item]), key=lambda x: abs(x[1]-fix[item]))

                indices[param_index] = value_index[0]

                del points[item]

            chisquare = chisquare[tuple(indices)]

        return points, chisquare
